/**
 * Copyright (c) 2017-2018 Thales Global Services S.A.S.
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     Abel Hegedus (IncQuery Labs Ltd.) - initial API and implementation
 */
package org.eclipse.emf.diffmerge.patch.persistence.emf;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Collections;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.diffmerge.patch.api.ModelPatch;
import org.eclipse.emf.diffmerge.patch.api.ModelPatchException;
import org.eclipse.emf.diffmerge.patch.persistence.emf.EMFModelPatchPersister;
import org.eclipse.emf.diffmerge.patch.persistence.emf.emodelpatch.EModelPatch;
import org.eclipse.emf.diffmerge.patch.serializer.IModelPatchSerializer;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ObjectExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

@SuppressWarnings("all")
public class EMFModelPatchSerializer implements IModelPatchSerializer {
  private final EMFModelPatchPersister persister = new EMFModelPatchPersister();
  
  @Override
  public ModelPatch load(final File file) throws ModelPatchException {
    try {
      final ResourceSetImpl rs = new ResourceSetImpl();
      final Resource r = rs.getResource(URI.createFileURI(file.getPath()), true);
      final EObject emodelpatch = IterableExtensions.<EObject>head(r.getContents());
      if ((emodelpatch instanceof EModelPatch)) {
        final ModelPatch modelpatch = this.persister.load(((EModelPatch)emodelpatch));
        return modelpatch;
      }
    } catch (final Throwable _t) {
      if (_t instanceof Exception) {
        final Exception e = (Exception)_t;
        throw new ModelPatchException("Exception occurred while loading model patch", e);
      } else {
        throw Exceptions.sneakyThrow(_t);
      }
    }
    return null;
  }
  
  @Override
  public ModelPatch load(final String source) throws ModelPatchException {
    try {
      InputStream stream = null;
      try {
        byte[] _bytes = source.getBytes(StandardCharsets.UTF_8);
        ByteArrayInputStream _byteArrayInputStream = new ByteArrayInputStream(_bytes);
        stream = _byteArrayInputStream;
        final InputStream reader = stream;
        final ResourceSetImpl rs = new ResourceSetImpl();
        Resource _createResource = rs.createResource(URI.createFileURI("file://dummy.emodelpatch"));
        final Procedure1<Resource> _function = (Resource it) -> {
          try {
            it.load(reader, Collections.<Object, Object>unmodifiableMap(CollectionLiterals.<Object, Object>newHashMap()));
          } catch (Throwable _e) {
            throw Exceptions.sneakyThrow(_e);
          }
        };
        final Resource r = ObjectExtensions.<Resource>operator_doubleArrow(_createResource, _function);
        final EObject emodelpatch = IterableExtensions.<EObject>head(r.getContents());
        if ((emodelpatch instanceof EModelPatch)) {
          final ModelPatch modelpatch = this.persister.load(((EModelPatch)emodelpatch));
          return modelpatch;
        }
      } catch (final Throwable _t) {
        if (_t instanceof Exception) {
          final Exception e = (Exception)_t;
          throw new ModelPatchException("Exception occurred while loading model patch", e);
        } else {
          throw Exceptions.sneakyThrow(_t);
        }
      } finally {
        stream.close();
      }
      return null;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  @Override
  public void serialize(final ModelPatch modelPatch, final File file) throws ModelPatchException {
    try {
      final ResourceSetImpl rs = new ResourceSetImpl();
      final Resource r = rs.createResource(URI.createFileURI(file.getPath()));
      final EModelPatch emodelpatch = this.persister.persist(modelPatch);
      r.getContents().add(emodelpatch);
      r.save(Collections.<Object, Object>unmodifiableMap(CollectionLiterals.<Object, Object>newHashMap()));
    } catch (final Throwable _t) {
      if (_t instanceof Exception) {
        final Exception e = (Exception)_t;
        throw new ModelPatchException("Exception occurred while loading model patch", e);
      } else {
        throw Exceptions.sneakyThrow(_t);
      }
    }
  }
  
  @Override
  public String serialize(final ModelPatch modelPatch) throws ModelPatchException {
    try {
      ByteArrayOutputStream stream = null;
      String result = null;
      try {
        final ResourceSetImpl rs = new ResourceSetImpl();
        final Resource r = rs.createResource(URI.createFileURI("file://dummy.emodelpatch"));
        final EModelPatch emodelpatch = this.persister.persist(modelPatch);
        r.getContents().add(emodelpatch);
        ByteArrayOutputStream _byteArrayOutputStream = new ByteArrayOutputStream();
        stream = _byteArrayOutputStream;
        r.save(stream, Collections.<Object, Object>unmodifiableMap(CollectionLiterals.<Object, Object>newHashMap()));
        result = stream.toString(StandardCharsets.UTF_8.name());
      } catch (final Throwable _t) {
        if (_t instanceof Exception) {
          final Exception e = (Exception)_t;
          throw new ModelPatchException("Exception occurred while loading model patch", e);
        } else {
          throw Exceptions.sneakyThrow(_t);
        }
      } finally {
        stream.close();
      }
      return result;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  @Override
  public String getPreferredFileExtension() {
    return "emodelpatch";
  }
}
