/**
 * Copyright (c) 2016-2018 Thales Global Services S.A.S.
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *   Abel Hegedus, Tamas Borbas, Balazs Grill, Peter Lunk, Daniel Segesdi (IncQuery Labs Ltd.) - initial API and implementation
 */
package org.eclipse.emf.diffmerge.patch.runtime.modelaccess;

import com.google.common.base.Objects;
import com.google.common.base.Optional;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.diffmerge.patch.api.ModelPatchException;
import org.eclipse.emf.diffmerge.patch.runtime.modelaccess.EMFModelAccess;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Exceptions;

@SuppressWarnings("all")
public abstract class AbstractEMFModleAccess implements EMFModelAccess {
  protected abstract EObject doCreate(final EClass type);
  
  protected abstract void doRemove(final EObject element);
  
  protected abstract void doAdd(final EObject container, final EStructuralFeature feature, final Object element);
  
  protected abstract void doAdd(final EObject container, final EStructuralFeature feature, final Object element, final int index);
  
  protected abstract void doRemove(final EObject container, final EStructuralFeature feature, final Object element);
  
  protected abstract void doRemove(final EObject container, final EStructuralFeature feature, final int index);
  
  protected abstract void doChangeIndex(final EObject container, final EStructuralFeature feature, final Object element, final int index);
  
  @Override
  public EObject create(final EClassifier type) {
    EObject _xifexpression = null;
    if ((type instanceof EClass)) {
      _xifexpression = this.doCreate(((EClass)type));
    } else {
      throw new IllegalArgumentException("Only EClasses are supported as element types.");
    }
    return _xifexpression;
  }
  
  @Override
  public void add(final EObject container, final EStructuralFeature feature, final Object element, final Optional<Integer> index) {
    try {
      boolean _isMany = feature.isMany();
      if (_isMany) {
        final Object featureValue = container.eGet(feature);
        if ((featureValue instanceof EList)) {
          if ((feature.isUnique() && ((EList)featureValue).contains(element))) {
            StringConcatenation _builder = new StringConcatenation();
            _builder.append("Element is already in the unique list");
            throw new ModelPatchException(_builder.toString());
          }
          if ((index.isPresent() && (((EList)featureValue).size() >= (index.get()).intValue()))) {
            this.doAdd(container, feature, element, (index.get()).intValue());
          } else {
            this.doAdd(container, feature, element);
          }
        } else {
          throw new IllegalStateException("Feature value is not EList");
        }
      } else {
        this.doAdd(container, feature, element);
      }
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  @Override
  public void remove(final EObject element) {
    this.doRemove(element);
  }
  
  @Override
  public void remove(final EObject container, final EStructuralFeature feature, final Object element, final Optional<Integer> index) {
    try {
      final Object featureValue = container.eGet(feature);
      boolean _isMany = feature.isMany();
      if (_isMany) {
        if ((featureValue instanceof EList)) {
          boolean _contains = ((EList)featureValue).contains(element);
          boolean _not = (!_contains);
          if (_not) {
            StringConcatenation _builder = new StringConcatenation();
            _builder.append("Cannot remove nonexistent element");
            throw new ModelPatchException(_builder.toString());
          }
          if (((index.isPresent() && (((EList)featureValue).size() > (index.get()).intValue())) && Objects.equal(((EList)featureValue).get((index.get()).intValue()), element))) {
            final int intIndex = (index.get()).intValue();
            this.doRemove(container, feature, intIndex);
          } else {
            this.doRemove(container, feature, element);
          }
        } else {
          throw new IllegalStateException("Feature value is not EList");
        }
      } else {
        boolean _equals = Objects.equal(featureValue, element);
        if (_equals) {
          this.doRemove(container, feature, element);
        } else {
          StringConcatenation _builder_1 = new StringConcatenation();
          _builder_1.append("Cannot remove nonexistent element");
          throw new ModelPatchException(_builder_1.toString());
        }
      }
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  @Override
  public void changeIndex(final EObject container, final EStructuralFeature feature, final Object element, final Optional<Integer> newIndex) {
    if ((feature.isMany() && newIndex.isPresent())) {
      final Object featureValue = container.eGet(feature);
      if ((featureValue instanceof EList)) {
        int _size = ((EList)featureValue).size();
        Integer _get = newIndex.get();
        boolean _greaterEqualsThan = (_size >= (_get).intValue());
        if (_greaterEqualsThan) {
          this.doChangeIndex(container, feature, element, (newIndex.get()).intValue());
        }
      } else {
        throw new IllegalStateException("Feature value is not EList");
      }
    }
  }
}
